"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.CompilerUserConfigSchema = exports.CompilerCliConfig = exports.DEFAULT_SRC_PATH = exports.PhantomDependencyMode = exports.CompressCssStrategies = exports.CSSMinimizerTypes = exports.JSMinimizerTypes = exports.GlobalObjectTransformTypes = exports.DevTools = exports.Modes = exports.CompileTypeDescriptions = exports.RootEntryConfigFiles = exports.CompileModes = exports.NODE_MODULE_REGEXP = exports.JSON_REGEXP = exports.STYLE_REGEXP = exports.SJS_REGEXP = exports.TEMPLATE_REGEXP = exports.SCRIPT_REGEXP = exports.TS_REGEXP = exports.SASS_REGEXP = exports.LESS_REGEXP = exports.AssetFileExtPattern = exports.AssetFileTypes = exports.AllFileTypes = exports.AllSjsFileTypes = exports.AllScriptFileTypes = exports.AllConfigFileTypes = exports.AllStyleFileTypes = exports.AllTemplateFileTypes = exports.ExtraConfigTypes = exports.ExtraScriptTypes = exports.ExtraStyleFileTypes = exports.NODE_MODULES = exports.DEFAULT_CHUNK_LOADING_GLOBAL = exports.NON_SCRIPT_ENTRIES_FILENAME = exports.MOR_RUNTIME_PACKAGE_REGEXP = exports.MOR_RUNTIME_NPMS = exports.RUNTIME_SOURCE_TYPES = exports.INDEPENDENT_SUBPACAKGE_JSON = exports.CHILD_COMPILER_RUNNER = exports.COMPILE_COMMAND_NAME = void 0;
const tslib_1 = require("tslib");
const utils_1 = require("@morjs/utils");
const path_1 = tslib_1.__importDefault(require("path"));
const compilerPlugins_1 = require("./compilerPlugins");
const utils_2 = require("./utils");
exports.COMPILE_COMMAND_NAME = 'compile';
/**
 * mor 子 runner 判断标识符，用于独立分包编译
 */
exports.CHILD_COMPILER_RUNNER = 'MOR_CHILD_RUNNER_COMPILER_FLAG';
/**
 * 独立分包 context key 名称
 */
exports.INDEPENDENT_SUBPACAKGE_JSON = 'independentSubpackageJson';
/**
 * 运行时源码类型标记
 */
exports.RUNTIME_SOURCE_TYPES = {
    alipay: 'a',
    wechat: 'w'
};
/**
 * mor 核心运行时基础库
 * 兼容旧的基础库
 */
exports.MOR_RUNTIME_NPMS = {
    api: ['@morjs/api', '@ali/openmor-api'],
    core: ['@morjs/core', '@ali/openmor-core']
};
/**
 * 生成 mor 运行时正则
 * 1. 用于实现转端运行时自动注入的检测
 * 2. 用于确保 API 转端处理时，跳过 MorJS 内部的代码
 * 注意: 这个正则必须包含两个捕获(即两对小括号), 第二个捕获字符串用于逻辑判断
 * 参见: plugins/runtimeInjectPlugin.ts 中的代码
 */
function generateMorRuntimeRegexp() {
    const rootDir = (0, utils_1.slash)(path_1.default.resolve(__dirname, '..'));
    let packages = exports.MOR_RUNTIME_NPMS.api
        .concat(exports.MOR_RUNTIME_NPMS.core)
        .concat('@morjs/runtime-base');
    // 代表为仓库代码
    if (rootDir.endsWith('packages/plugin-compiler')) {
        packages = packages.concat([
            rootDir.replace(/\//g, '\\/').replace(/plugin-compiler$/, 'core'),
            rootDir.replace(/\//g, '\\/').replace(/plugin-compiler$/, 'api'),
            rootDir.replace(/\//g, '\\/').replace(/plugin-compiler$/, 'runtime-base')
        ]);
    }
    return new RegExp(`(${packages.join('|')})\/(?:lib|esm)\/(app|page|component|api|env).js$`);
}
/**
 * mor 运行时正则匹配
 */
exports.MOR_RUNTIME_PACKAGE_REGEXP = generateMorRuntimeRegexp();
// 非 js/ts 文件会统一使用一个 entry 来囊括
exports.NON_SCRIPT_ENTRIES_FILENAME = 'mor_non_script_entries';
// 默认的 jsonpFunction
exports.DEFAULT_CHUNK_LOADING_GLOBAL = 'mor_modules';
exports.NODE_MODULES = 'node_modules';
// 额外支持的样式文件
exports.ExtraStyleFileTypes = [
    // less 支持
    '.less',
    // sass 支持
    '.scss',
    '.sass'
];
// 额外支持的脚本文件
exports.ExtraScriptTypes = [
    '.mjs',
    // ts 支持
    '.ts',
    '.mts'
];
// 额外支持的配置文件
exports.ExtraConfigTypes = [
    // jsonc 支持
    '.jsonc',
    '.json5'
];
// 支持的各类文件类型, 以小程序本身文件为主
exports.AllTemplateFileTypes = [...compilerPlugins_1.PluginTemplateFileTypes];
exports.AllStyleFileTypes = [
    ...compilerPlugins_1.PluginStyleFileTypes,
    ...exports.ExtraStyleFileTypes
];
exports.AllConfigFileTypes = [
    ...compilerPlugins_1.PluginConfigFileTypes,
    ...exports.ExtraConfigTypes
];
exports.AllScriptFileTypes = [
    ...compilerPlugins_1.PluginScriptFileTypes,
    ...exports.ExtraScriptTypes
];
exports.AllSjsFileTypes = [...compilerPlugins_1.PluginSjsFileTypes];
exports.AllFileTypes = [
    ...exports.AllScriptFileTypes,
    ...exports.AllConfigFileTypes,
    ...exports.AllTemplateFileTypes,
    ...exports.AllStyleFileTypes,
    ...exports.AllSjsFileTypes
];
// 支持的拷贝的资源文件类型
exports.AssetFileTypes = [
    '.jpg',
    '.jpeg',
    '.png',
    '.svg',
    '.bmp',
    '.ico',
    '.gif',
    '.webp',
    '.otf',
    '.ttf',
    '.woff',
    '.woff2',
    '.eot',
    '.cer',
    '.ogg',
    '.aac',
    '.mp4',
    '.wav',
    '.mp3',
    '.m4a',
    '.silk',
    '.wasm',
    '.br',
    '.cert'
];
// 资源文件 glob pattern
exports.AssetFileExtPattern = (0, utils_2.extsToGlobPatterns)(exports.AssetFileTypes);
// 一些 正则
exports.LESS_REGEXP = /\.less$/;
exports.SASS_REGEXP = /\.s(a|c)ss$/;
exports.TS_REGEXP = /\.m?ts$/;
exports.SCRIPT_REGEXP = /\.m?js$/;
exports.TEMPLATE_REGEXP = /\.(wx|ax)ml$/;
exports.SJS_REGEXP = /\.(sjs|wxs)$/;
exports.STYLE_REGEXP = /\.(wx|ac)ss$/;
exports.JSON_REGEXP = /\.json(c|5)?$/;
exports.NODE_MODULE_REGEXP = /[\\/]node_modules[\\/]/;
/**
 * 编译模式
 * - bundle: 打包编译模式, 会处理 npm 包并将代码按照特定逻辑进行打包, 产物中无 node_modules
 * - transform: 转换编译模式, 仅转换和编译源码，不处理 npm 组件和依赖（由 default 和 transfer 合并而来）
 * - NOTE: transfer 和 default 模式已合并为 transform, 出于兼容性考虑这里依然替用配置支持
 */
exports.CompileModes = (0, utils_1.objectEnum)([
    'bundle',
    'transform',
    'transfer',
    'default'
]);
/**
 * 入口配置文件类型
 * - app.json: 小程序入口配置文件
 * - plugin.json: 小程序插件入口配置文件
 * - subpackage.json: 小程序分包入口配置文件
 */
exports.RootEntryConfigFiles = (0, utils_1.objectEnum)([
    'app.json',
    'plugin.json',
    'subpackage.json'
]);
/**
 * 编译类型描述
 */
exports.CompileTypeDescriptions = {
    miniprogram: '小程序',
    plugin: '插件',
    subpackage: '分包',
    component: '组件'
};
/**
 * 构建模式, 和 webpack 一致
 */
exports.Modes = (0, utils_1.objectEnum)(['production', 'development', 'none']);
/**
 * devtools 和 webpack 一致
 * 参见: https://webpack.js.org/configuration/devtool/
 */
exports.DevTools = (0, utils_1.objectEnum)([
    'eval',
    'eval-cheap-source-map',
    'eval-cheap-module-source-map',
    'eval-source-map',
    'cheap-source-map',
    'cheap-module-source-map',
    'source-map',
    'inline-cheap-source-map',
    'inline-cheap-module-source-map',
    'inline-source-map',
    'eval-nosources-cheap-source-map',
    'eval-nosources-cheap-module-source-map',
    'eval-nosources-source-map',
    'inline-nosources-cheap-source-map',
    'inline-nosources-cheap-module-source-map',
    'inline-nosources-source-map',
    'nosources-cheap-source-map',
    'nosources-cheap-module-source-map',
    'nosources-source-map',
    'hidden-nosources-cheap-source-map',
    'hidden-nosources-cheap-module-source-map',
    'hidden-nosources-source-map',
    'hidden-cheap-source-map',
    'hidden-cheap-module-source-map',
    'hidden-source-map'
]);
/**
 * 全局对象替换方式
 * - `enhanced` 增强方式: wx => mor 并追加 mor import, mor 中包含 api 的抹平
 * - `lite` 轻量级的方式: wx => my, 替换所有 globalObject
 * - `minimal` 最小替换: wx.abc() => my.abc(), 仅替换函数调用
 */
exports.GlobalObjectTransformTypes = (0, utils_1.objectEnum)([
    'enhanced',
    'lite',
    'minimal'
]);
// 依赖分析相关
const AnalyzerModes = (0, utils_1.objectEnum)(['server', 'static', 'json', 'disabled']);
const AnalyzerDefaultSizes = (0, utils_1.objectEnum)(['stat', 'parsed', 'gzip']);
const AnalyzerLogLevel = (0, utils_1.objectEnum)(['info', 'warn', 'error', 'silent']);
// JS 压缩选项
exports.JSMinimizerTypes = (0, utils_1.objectEnum)(['terser', 'esbuild', 'swc']);
// CSS 压缩选项
exports.CSSMinimizerTypes = (0, utils_1.objectEnum)([
    'esbuild',
    'csso',
    'cssnano',
    'cleancss',
    'parcelcss'
]);
// CSS 压缩特性
exports.CompressCssStrategies = (0, utils_1.objectEnum)(['lite']);
// 幽灵依赖检测模式
exports.PhantomDependencyMode = (0, utils_1.objectEnum)(['warn', 'error']);
exports.DEFAULT_SRC_PATH = './src';
// 编译配置
exports.CompilerCliConfig = {
    /**
     * 源码类型；
     * 默认可不配置, JS 文件会自动基于 .wxml 或 .axml 的后缀名来判断是哪个平台的页面或组件
     * 如果一个文件夹下面同时出现了这两种文件, 需要手动指定该类型来帮助判断
     */
    sourceType: {
        cliOption: '--source-type <sourceType>',
        name: '源码类型',
        overwriteUserConfig: true,
        desc: '用于判断小程序页面或组件使用了哪种 DSL',
        valuesDesc() {
            return (0, utils_1.validKeysMessage)(utils_1.SourceTypes);
        }
    },
    /**
     * 编译目标
     * alipay: 支付宝小程序
     * wechat: 微信小程序
     * baidu: 百度小程序
     * qq: QQ 小程序
     * bytedance: 字节小程序
     * web: Web 应用
     */
    target: {
        cliOption: '-t, --target <target>',
        name: `编译目标`,
        overwriteUserConfig: true,
        desc: '将当前的工程编译为目标小程序工程',
        valuesDesc() {
            return (0, utils_1.validKeysMessage)((0, compilerPlugins_1.getAllCompilerTargets)());
        }
    },
    /**
     * 编译模式, 默认为 bundle
     * 另外支持 transform 模式
     */
    compileMode: {
        cliOption: '--compile-mode <compileMode>',
        name: '编译模式',
        overwriteUserConfig: true,
        desc: '将当前工程以指定的编译模式编译, 编译模式差异参见官方文档',
        valuesDesc() {
            return (0, utils_1.validKeysMessage)(exports.CompileModes);
        }
    },
    /**
     * 编译的形态, 支持 miniprogram, plugin, subpackage, 默认为 miniprogram
     * 不同形态的入口文件不一样
     *   miniprogram: app.json
     *   plugin: plugin.json
     *   subpackage: subpackage.json
     */
    compileType: {
        cliOption: '--compile-type <compileType>',
        name: '编译形态',
        overwriteUserConfig: true,
        desc: '将当前工程编译为指定形态',
        valuesDesc() {
            return (0, utils_1.validKeysMessage)(utils_1.CompileTypes);
        }
    },
    // 关闭 devtool
    noDevtool: {
        cliOption: '--no-devtool',
        overwriteUserConfig: true,
        name: '关闭 devtool'
    },
    devtool: {
        cliOption: '-d, --devtool [devtool]',
        name: '开发工具',
        overwriteUserConfig: true,
        desc: '控制是否生成, 以及如何生成 source map, 参见 https://webpack.js.org/configuration/devtool',
        default: null
    },
    // 是否开启压缩, mode 为 production 时, 自动为 true, 默认为 false
    minimize: {
        cliOption: '--minimize',
        name: '是否开启压缩',
        overwriteUserConfig: true,
        desc: '--production 状态下会自动开启 (默认: false)'
    },
    // 关闭 JS 压缩
    noJsMinimizer: {
        cliOption: '--no-js-minimizer',
        name: '关闭 JS 压缩'
    },
    // JS 压缩器
    jsMinimizer: {
        cliOption: '--js-minimizer [minimizer]',
        name: 'JS 代码压缩器',
        overwriteUserConfig: true,
        default: null,
        valuesDesc() {
            return (0, utils_1.validKeysMessage)(exports.JSMinimizerTypes);
        }
    },
    // 关闭 CSS 压缩
    noCssMinimizer: {
        cliOption: '--no-css-minimizer',
        name: '关闭 CSS 压缩'
    },
    // CSS 压缩器
    cssMinimizer: {
        cliOption: '--css-minimizer [minimizer]',
        name: 'CSS 代码压缩器, 默认为 esbuild',
        overwriteUserConfig: true,
        default: null,
        valuesDesc() {
            return (0, utils_1.validKeysMessage)(exports.CSSMinimizerTypes);
        }
    },
    // 关闭 XML 压缩
    noXmlMinimizer: {
        cliOption: '--no-xml-minimizer',
        name: '关闭 XML 压缩'
    },
    // XML 压缩器
    xmlMinimizer: {
        cliOption: '--xml-minimizer',
        name: 'XML 代码压缩器, 目前仅支持 html-terser',
        overwriteUserConfig: true,
        default: null
    },
    // 和 webpack mode 意义保持一致
    mode: {
        cliOption: '--mode <mode>',
        name: '开发模式',
        overwriteUserConfig: true,
        desc: '设置开发模式',
        valuesDesc() {
            return (0, utils_1.validKeysMessage)(exports.Modes);
        }
    },
    // 是否开启生产模式
    production: {
        cliOption: '--production',
        name: '是否开启生产模式',
        desc: '等同于 --mode production'
    },
    // 是否自动清空输出目录
    autoClean: {
        cliOption: '--auto-clean',
        name: '是否自动清空输出目录',
        overwriteUserConfig: true,
        desc: '(默认: false)'
    },
    // 是否开启 watch 模式
    watch: {
        cliOption: '-w, --watch',
        name: '是否开启监听模式',
        overwriteUserConfig: true,
        desc: '(默认: false)'
    },
    // 源代码根目录
    srcPath: {
        cliOption: '-s, --src-path <dir>',
        name: '源代码根目录, 默认为 src',
        overwriteUserConfig: true
    },
    // 编译产物输出目录
    // 不同的 target 会有默认的输出目录
    outputPath: {
        cliOption: '-o, --output-path <dir>',
        name: '编译产物输出目录',
        overwriteUserConfig: true,
        desc: '不同的 target 会有默认的输出目录, 如 dist/wechat'
    },
    // 忽略文件或目录
    ignore: {
        cliOption: '--ignore <fileOrDir>',
        name: '忽略文件或目录',
        overwriteUserConfig: true,
        desc: '各个配置中的 outputPath 会被自动添加到忽略目录'
    },
    // 是否关闭缓存
    noCache: {
        cliOption: '--no-cache',
        name: '是否关闭缓存'
    },
    // 是否开启缓存
    cache: {
        cliOption: '--cache',
        name: '是否开启缓存',
        overwriteUserConfig: true,
        desc: 'mode = development 下默认开启, mode = production 状态下默认关闭',
        default: null
    },
    // 是否开启缓存
    processNodeModules: {
        cliOption: '--process-node-modules',
        overwriteUserConfig: true,
        name: '是否自动处理 node_modules 中的多端组件库',
        desc: '默认情况为 false, 开启后会自动处理 node_modules 中的文件的转端'
    },
    // 全局对象
    globalObject: {
        cliOption: '--global-object <name>',
        name: '全局对象配置',
        overwriteUserConfig: true,
        desc: '不同的 target 会有默认的全局对象, 通常情况下无需设置'
    },
    // 是否自动注入 runtime
    autoInjectRuntime: {
        name: '是否自动注入 mor 运行时',
        children: {
            app: {
                name: 'App 运行时注入, 编译时替换 App({}) 为 mor 的运行时',
                desc: '跨端编译时, 默认为: true'
            },
            page: {
                name: 'Page 运行时注入, 编译时替换 Page({}) 为 mor 的运行时',
                desc: '跨端编译时, 默认为: true'
            },
            component: {
                name: 'Component 运行时注入, 编译时替换 Component({}) 为 mor 的运行时',
                desc: '跨端编译时, 默认为: true'
            },
            behavior: {
                name: 'Behavior 运行时注入, 编译时替换 Behavior({}) 为 mor 的运行时',
                desc: '跨端编译且源码类型非支付宝小程序时, 默认为: true'
            },
            mixin: {
                name: 'Mixin 运行时注入, 编译时替换 Mixin({}) 为 mor 的运行时',
                desc: '跨端编译且源码类型为支付宝小程序时, 默认为: true'
            },
            api: {
                name: 'API 运行时注入, 全局对象替换方式',
                valuesDesc() {
                    return (0, utils_1.validKeysMessage)(exports.GlobalObjectTransformTypes) + '或 false';
                }
            }
        }
    },
    // 是否开启幽灵依赖检查
    phantomDependency: {
        name: '是否开启幽灵依赖检查',
        children: {
            mode: {
                name: '开启幽灵依赖检查的模式',
                desc: '不同模式下幽灵依赖检查的程度不同'
            },
            exclude: {
                name: '不作为幽灵依赖的 npm 包',
                desc: '开启幽灵依赖检查时，不被认为是幽灵依赖的 npm 包'
            }
        }
    },
    // 模拟 app.x 文件
    mockAppEntry: {
        name: '模拟 app.x 的文件配置',
        desc: '需要结合 compileType 使用, 默认情况下 均为 app, ' +
            '`plugin` 编译条件下, 优先使用 mor.plugin.app, ' +
            '`subpackage` 编译条件下, 优先使用 mor.subpackage.app'
    },
    // ts 编译配置, 和 tsconfig 中的含义一致
    // 剩余 tsconfig 的配置子集
    compilerOptions: {
        name: 'ts 编译配置',
        children: {
            declaration: {
                name: '是否生成 declaration (.d.ts) 文件',
                desc: '仅 compileMode 为 default 情况下支持'
            },
            importHelpers: {
                name: '是否引入 tslib',
                desc: '需要添加 tslib 后才能生效'
            },
            module: {
                name: '模块输出类型',
                desc: '不同的小程序会有不同的默认值',
                valuesDesc() {
                    return (0, utils_1.validKeysMessage)(utils_1.CompileModuleKind);
                }
            },
            target: {
                name: '输出的 ES 版本',
                desc: '不同的小程序会有不同的默认值',
                valuesDesc() {
                    return (0, utils_1.validKeysMessage)(utils_1.CompileScriptTarget);
                }
            },
            esModuleInterop: {
                name: '是否开启 ES 模块互操作性'
            },
            allowSyntheticDefaultImports: {
                name: '是否允许合成默认导入'
            }
        }
    },
    // 条件编译配置
    conditionalCompile: {
        name: '条件编译配置',
        children: {
            context: {
                name: '代码纬度条件编译的上下文'
            },
            fileExt: {
                name: '文件纬度条件编译的后缀设置',
                desc: '不同的 target 有一套默认的配置'
            }
        }
    },
    // 开启 bundle analyze
    // 选项和 webpack-bundle-analyzer 选项一一对应
    analyzer: {
        name: '是否开启 bundle analyzer',
        cliOption: '--analyze',
        children: {
            analyzerMode: {
                name: '依赖分析模式'
            },
            analyzerHost: {
                name: '依赖分析 HTTP 服务域名',
                desc: '仅在 analyzerMode 为 server 时生效'
            },
            analyzerPort: {
                name: '依赖分析 HTTP 服务端口号',
                desc: '仅在 analyzerMode 为 server 时生效'
            },
            reportFilename: {
                name: '生成报告文件的名称',
                desc: '仅在 analyzerMode 为 static 时生效'
            },
            reportTitle: {
                name: '生成报告文件的标题',
                desc: '仅在 analyzerMode 为 static 时生效'
            },
            defaultSizes: {
                name: '分析报告中展示模块大小的定义方式'
            },
            openAnalyzer: {
                name: '是否在浏览器中打开'
            },
            generateStatsFile: {
                name: '是否生成 stats 文件'
            },
            statsFilename: {
                name: 'stats 文件名称'
            },
            // 参见: https://webpack.js.org/configuration/stats/
            statsOptions: {
                name: 'stats 生成内容选项'
            },
            excludeAssets: {
                name: '排除 assets 配置'
            },
            logLevel: {
                name: '日志级别'
            }
        }
    },
    // 实验特性
    experiments: {
        name: '实验特性',
        children: {
            // 自动裁剪 ts 和 babel 冗余的辅助函数功能
            autoTrimRuntimeHelpers: {
                name: '自动裁剪 ts 和 babel 冗余的辅助函数',
                desc: '开启后需要项目中手动添加 tslib 和 @babel/runtime 的依赖'
            },
            // CSS 类名压缩功能
            compressCssClassName: {
                name: 'CSS 类名压缩功能',
                children: {
                    strategy: {
                        name: '压缩策略',
                        desc: '目前仅支持 轻量级(lite), lite 模式下, 会跳过 app.acss ' +
                            '和 包含 {{}} 的 axml 以及对应的 acss 文件',
                        valuesDesc() {
                            return (0, utils_1.validKeysMessage)(exports.CompressCssStrategies);
                        }
                    },
                    // 压缩后的前缀, 默认为 ''
                    prefix: {
                        name: '压缩后的 class 前缀'
                    },
                    surfix: {
                        name: '压缩后的 class 后缀'
                    },
                    include: {
                        name: '文件过滤包含规则'
                    },
                    exclude: {
                        name: '文件过滤排除规则'
                    },
                    // 一组不需要重命名的 class 名称, 可以将不需要重命名的 class 放在这里
                    except: {
                        name: '不需要重命名的 class 名称',
                        desc: '可以将不需要重命名的 class 放在这里'
                    },
                    // 用于生成随机类名的字母库, 默认为:
                    // _abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789
                    alphabet: {
                        name: '用于生成随机类名的字母库'
                    },
                    customAttributeNames: {
                        name: '自定义属性名称',
                        desc: '用于指定一些自定义的 class 名称, 比如 innerClass 等, ' +
                            '配置的自定义 class 属性会被当做 class 同样被处理'
                    },
                    // 类名过滤, 支持配置自定义函数, 返回 true 代表可以重命名, false 代表不可以重命名
                    // (className: string, filePath: string) => boolean
                    classNameFilter: {
                        name: '自定义类名过滤方法',
                        desc: '支持配置自定义函数, 返回 true 代表可以重命名, false 代表不可以重命名'
                    },
                    // 动态类名检测关闭逻辑，支持自定义函数 或 boolean，返回 true 表明即使模版中存在动态绑定 class，仍然对当前模板进行类名压缩
                    // 返回 false 表明走默认的存在动态绑定 class 不压缩的逻辑
                    // (filePath: string) => boolean
                    disableDynamicClassDetection: {
                        name: '关闭动态类名检测',
                        desc: '支持布尔值和支持配置自定义函数。如果设置为函数，返回 true 代表存在动态类名绑定时可以走类名压缩， 返回 false 表明走默认的存在动态绑定 class 不压缩的逻辑'
                    },
                    // 处理完成回调, 可获取 类名映射
                    // (classNameMappings: Map<string, string>) => void
                    success: {
                        name: '处理完成回调',
                        desc: '可用于获取类名映射的结果'
                    }
                }
            }
        }
    }
};
const compressCssClassNameSchema = utils_1.zod
    .object({
    strategy: utils_1.zod
        .nativeEnum(exports.CompressCssStrategies)
        .optional()
        .default(exports.CompressCssStrategies.lite),
    prefix: utils_1.zod.string().optional().default(''),
    surfix: utils_1.zod.string().optional().default(''),
    include: utils_1.zod.array(utils_1.zod.instanceof(RegExp)).optional().default([]),
    exclude: utils_1.zod.array(utils_1.zod.instanceof(RegExp)).optional().default([]),
    except: utils_1.zod.array(utils_1.zod.string()).optional().default([]),
    alphabet: utils_1.zod
        .string()
        .optional()
        .default('_abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789'),
    customAttributeNames: utils_1.zod.array(utils_1.zod.string()).optional().default([]),
    classNameFilter: utils_1.zod
        .function()
        .args(utils_1.zod.string(), utils_1.zod.string())
        .returns(utils_1.zod.boolean())
        .optional(),
    disableDynamicClassDetection: utils_1.zod
        .function()
        .args(utils_1.zod.string())
        .returns(utils_1.zod.boolean())
        .or(utils_1.zod.boolean())
        .optional(),
    success: utils_1.zod
        .function()
        .args(utils_1.zod.map(utils_1.zod.string(), utils_1.zod.string()))
        .returns(utils_1.zod.void())
        .optional()
})
    .optional()
    .or(utils_1.zod.boolean().optional())
    .default(false);
// bundle 依赖分析 schema
const analyzerExcludeAssetsSchema = utils_1.zod
    .string()
    .or(utils_1.zod.instanceof(RegExp).optional())
    .or(utils_1.zod.function().args(utils_1.zod.string()).returns(utils_1.zod.boolean()));
const analyzerSchema = utils_1.zod
    .object({
    analyzerMode: utils_1.zod.nativeEnum(AnalyzerModes).default(AnalyzerModes.server),
    analyzerHost: utils_1.zod.string().default('127.0.0.1'),
    analyzerPort: utils_1.zod
        .string()
        .regex(/^[0-9]+$/)
        .transform((v) => parseInt(v))
        .or(utils_1.zod.number())
        .or(utils_1.zod.nativeEnum((0, utils_1.objectEnum)(['auto'])))
        .default(8888),
    reportFilename: utils_1.zod.string().default('report.html'),
    reportTitle: utils_1.zod
        .string()
        .optional()
        .or(utils_1.zod.function().returns(utils_1.zod.string()).optional()),
    defaultSizes: utils_1.zod
        .nativeEnum(AnalyzerDefaultSizes)
        .optional()
        .default(AnalyzerDefaultSizes.parsed),
    openAnalyzer: utils_1.zod.boolean().optional().default(true),
    generateStatsFile: utils_1.zod.boolean().optional().default(false),
    statsFilename: utils_1.zod.string().default('stats.json'),
    statsOptions: utils_1.zod.any().optional(),
    excludeAssets: utils_1.zod
        .array(analyzerExcludeAssetsSchema)
        .or(analyzerExcludeAssetsSchema)
        .optional(),
    logLevel: utils_1.zod
        .nativeEnum(AnalyzerLogLevel)
        .optional()
        .default(AnalyzerLogLevel.info)
})
    .or(utils_1.zod.boolean())
    .optional();
// 自定义用户配置, 和 CompilerCliConfig 基本上是一一对应的, 个别仅用于命令行的除外
exports.CompilerUserConfigSchema = utils_1.zod.object({
    sourceType: utils_1.zod.nativeEnum(utils_1.SourceTypes).default(utils_1.SourceTypes.wechat),
    target: utils_1.zod.nativeEnum((0, utils_1.objectEnum)(compilerPlugins_1.Targets)).default(compilerPlugins_1.Targets[0]),
    compileMode: utils_1.zod.nativeEnum(exports.CompileModes).default(exports.CompileModes.bundle),
    compileType: utils_1.zod.nativeEnum(utils_1.CompileTypes).default(utils_1.CompileTypes.miniprogram),
    devtool: utils_1.zod.nativeEnum(exports.DevTools).optional().or(utils_1.zod.boolean().optional()),
    minimize: utils_1.zod.boolean().optional(),
    jsMinimizer: utils_1.zod.nativeEnum(exports.JSMinimizerTypes).or(utils_1.zod.boolean()).default(true),
    jsMinimizerOptions: utils_1.zod.record(utils_1.zod.any()).optional(),
    cssMinimizer: utils_1.zod
        .nativeEnum(exports.CSSMinimizerTypes)
        .or(utils_1.zod.boolean())
        .default(exports.CSSMinimizerTypes.esbuild),
    cssMinimizerOptions: utils_1.zod.record(utils_1.zod.any()).optional(),
    xmlMinimizer: utils_1.zod.boolean().optional(),
    xmlMinimizerOptions: utils_1.zod.record(utils_1.zod.any()).optional(),
    mode: utils_1.zod.nativeEnum(exports.Modes).default(exports.Modes.development),
    srcPath: utils_1.zod.string().default('./src'),
    srcPaths: utils_1.zod.array(utils_1.zod.string()).optional(),
    outputPath: utils_1.zod.string().optional(),
    autoClean: utils_1.zod.boolean().optional().default(false),
    watch: utils_1.zod.boolean().optional().default(false),
    ignore: utils_1.zod.array(utils_1.zod.string()).default([]),
    cache: utils_1.zod.boolean().optional(),
    globalObject: utils_1.zod.string().optional(),
    /**
     * 用于配置全局文件的后缀，避免冲突，以及 globalChunkLoading 的后缀添加
     */
    globalNameSuffix: utils_1.zod.string().optional(),
    /**
     * 自定义 entries
     * 1. 用于自定义 app.json / subpackage.json / plugin.json 等入口文件
     * 2. 用于配置 额外需要生成的入口文件，如某个期望在 bundle 后依然能保持正确的路径的文件
     * 3. bundle 模式下，无引用关系，但需要额外需要编译的 页面（pages） 或 组件（components）
     */
    customEntries: utils_1.zod
        .object({
        'app.json': utils_1.zod.string().optional(),
        'subpackage.json': utils_1.zod.string().optional(),
        'plugin.json': utils_1.zod.string().optional(),
        'component.json': utils_1.zod.string().optional(),
        pages: utils_1.zod.array(utils_1.zod.string()).optional(),
        components: utils_1.zod.array(utils_1.zod.string()).optional()
    })
        .passthrough()
        .optional(),
    autoInjectRuntime: utils_1.zod
        .object({
        app: utils_1.zod.boolean().optional(),
        page: utils_1.zod.boolean().optional(),
        component: utils_1.zod.boolean().optional(),
        behavior: utils_1.zod.boolean().optional(),
        mixin: utils_1.zod.boolean().optional(),
        api: utils_1.zod.boolean().optional().or(utils_1.zod.nativeEnum(exports.GlobalObjectTransformTypes))
    })
        .or(utils_1.zod.boolean())
        .default(true),
    phantomDependency: utils_1.zod
        .object({
        mode: utils_1.zod
            .nativeEnum(exports.PhantomDependencyMode)
            .default(exports.PhantomDependencyMode.warn),
        exclude: utils_1.zod.array(utils_1.zod.string()).optional()
    })
        .or(utils_1.zod.boolean())
        .default(false),
    compilerOptions: utils_1.zod
        .object({
        /**
         * 用于自动矫正 commonjs 和 esm 混用的情况
         * 仅当 module 不是 commonjs 且 importHelpers 开启时生效
         * 原因为: typescript 引入 importHelpers 的时候会根据 设定的 module 来决定
         * 是用 esm 还是 commonjs 语法
         * 可能会导致 esm 和 commonjs 混用而引起编译问题
         */
        autoCorrectModuleKind: utils_1.zod.boolean().optional(),
        declaration: utils_1.zod.boolean().optional().default(false),
        importHelpers: utils_1.zod.boolean().optional(),
        module: utils_1.zod.nativeEnum(utils_1.CompileModuleKind).optional(),
        target: utils_1.zod.nativeEnum(utils_1.CompileScriptTarget).optional(),
        esModuleInterop: utils_1.zod.boolean().optional(),
        allowSyntheticDefaultImports: utils_1.zod.boolean().optional()
    })
        .optional()
        .default({}),
    conditionalCompile: utils_1.zod
        .object({
        context: utils_1.zod.record(utils_1.zod.any()).optional().default({}),
        fileExt: utils_1.zod.string().or(utils_1.zod.array(utils_1.zod.string())).optional()
    })
        .optional()
        .default({}),
    mockAppEntry: utils_1.zod.string().optional(),
    experiments: utils_1.zod
        .object({
        autoTrimRuntimeHelpers: utils_1.zod.boolean().default(false).optional(),
        compressCssClassName: compressCssClassNameSchema
    })
        .optional(),
    analyzer: analyzerSchema,
    /**
     * 支持文件拷贝
     * 支持 ['file1', 'file2'] 或 [{ from: 'abc/2', to: 'cde/2' }]
     * 其中 from 是相对于 srcPath 或 srcPaths 的路径
     *     to 是相对于 outputPath 的路径
     */
    copy: utils_1.zod
        .array(utils_1.zod.string().or(utils_1.zod.object({
        from: utils_1.zod.string(),
        to: utils_1.zod.string(),
        context: utils_1.zod.string().optional(),
        force: utils_1.zod.boolean().optional(),
        globOptions: utils_1.zod.record(utils_1.zod.any()).optional(),
        priority: utils_1.zod.number().optional(),
        toType: utils_1.zod.string().optional(),
        filter: utils_1.zod.function().args(utils_1.zod.string()).returns(utils_1.zod.boolean()).optional()
    })))
        .optional(),
    // 是否处理 node_modules 中的组件, 缺省状态下, 会基于不同的 target 自动选择是否处理
    processNodeModules: utils_1.zod
        .boolean()
        .or(utils_1.zod.object({
        include: utils_1.zod
            .instanceof(RegExp)
            .or(utils_1.zod.array(utils_1.zod.instanceof(RegExp)))
            .default([]),
        exclude: utils_1.zod
            .instanceof(RegExp)
            .or(utils_1.zod.array(utils_1.zod.instanceof(RegExp)))
            .default([])
    }))
        .optional(),
    /**
     * 配置代码中的变量
     * 目前仅 bundle 模式下生效
     */
    define: utils_1.zod.record(utils_1.zod.any()).default({}),
    /**
     * 配置别名
     */
    alias: utils_1.zod.record(utils_1.zod.any()).default({}),
    /**
     * 是否处理 componentPlaceholder 中的组件
     * compileType 为 miniprogram 或 plugin 时默认为 true
     * compileType 为 subpackage 或 component 时默认为 false
     */
    processPlaceholderComponents: utils_1.zod.boolean().optional(),
    /**
     * 是否处理组件入参函数
     */
    processComponentsPropsFunction: utils_1.zod.boolean().optional().default(false),
    /**
     * 配置可以共享的 node_modules 模块, 通常用于主子分包分仓库管理集成的场景
     */
    shared: utils_1.zod.array(utils_1.zod.string().or(utils_1.zod.record(utils_1.zod.string()))).default([]),
    /**
     * 配置需要消费的 node_modules 模块, 通常用于主子分包分仓库管理集成的场景
     */
    consumes: utils_1.zod.array(utils_1.zod.string().or(utils_1.zod.record(utils_1.zod.string()))).default([]),
    /**
     * 是否生成 mor.p.js 文件，用于更新集成时 app.json 内容
     */
    generateAppJSONScript: utils_1.zod.boolean().default(true)
});
//# sourceMappingURL=constants.js.map