"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.ProcessComponentsPropsFunctionPlugin = void 0;
const tslib_1 = require("tslib");
const utils_1 = require("@morjs/utils");
const path_1 = tslib_1.__importDefault(require("path"));
/**
 * 处理组件入参函数
 */
class ProcessComponentsPropsFunctionPlugin {
    constructor() {
        this.name = 'ProcessComponentsPropsFunctionPlugin';
        this.entryBuilder = null;
        this.filePathIdentifier = 'mor';
        this.needProcessList = null;
        this.processPropsObj = null;
        this.needProcessList = [];
        this.processPropsObj = {};
    }
    apply(runner) {
        runner.hooks.entryBuilder.tap(this.name, (entryBuilder) => {
            this.entryBuilder = entryBuilder;
        });
        runner.hooks.beforeRun.tap(this.name, () => {
            var _a, _b, _c;
            // 仅对开启了 processComponentsPropsFunction 为 true，支付宝转其他端的情况
            if (((_a = runner.userConfig) === null || _a === void 0 ? void 0 : _a.processComponentsPropsFunction) &&
                ((_b = runner.userConfig) === null || _b === void 0 ? void 0 : _b.sourceType) === utils_1.SourceTypes.alipay &&
                ((_c = runner.userConfig) === null || _c === void 0 ? void 0 : _c.target) !== utils_1.SourceTypes.alipay) {
                // 筛选出所有组件的 script 文件，通过 ast 判断该文件是否用到了 props 传递函数
                runner.hooks.addEntry.tap(this.name, (entryInfo) => {
                    var _a, _b, _c, _d;
                    if (((_a = entryInfo === null || entryInfo === void 0 ? void 0 : entryInfo.entry) === null || _a === void 0 ? void 0 : _a.entryType) === utils_1.EntryType.component &&
                        ((_b = entryInfo === null || entryInfo === void 0 ? void 0 : entryInfo.entry) === null || _b === void 0 ? void 0 : _b.entryFileType) === utils_1.EntryFileType.script) {
                        const sourceText = utils_1.fsExtra.readFileSync((_c = entryInfo === null || entryInfo === void 0 ? void 0 : entryInfo.entry) === null || _c === void 0 ? void 0 : _c.fullPath, 'utf-8') || '';
                        if (sourceText.includes('props')) {
                            const sourceFile = utils_1.typescript.createSourceFile(((_d = entryInfo === null || entryInfo === void 0 ? void 0 : entryInfo.entry) === null || _d === void 0 ? void 0 : _d.relativePath) || '__temp.ts', sourceText, utils_1.typescript.ScriptTarget.ESNext, true);
                            const visitNode = (node) => {
                                var _a, _b;
                                if (utils_1.typescript.isPropertyAssignment(node) &&
                                    utils_1.typescript.isIdentifier(node.name) &&
                                    node.name.escapedText === 'props' &&
                                    utils_1.typescript.isObjectLiteralExpression(node.initializer)) {
                                    const nodePropsList = node.initializer.properties;
                                    const [propsNormalList, propsFunctionList] = [[], []];
                                    const [propsNormalHandlers, propsMorHandlers] = [{}, {}];
                                    for (const item of nodePropsList) {
                                        if (utils_1.typescript.isPropertyAssignment(item) &&
                                            utils_1.typescript.isIdentifier(item.name) &&
                                            (utils_1.typescript.isArrowFunction(item.initializer) ||
                                                utils_1.typescript.isFunctionExpression(item.initializer))) {
                                            const escapedText = String(item.name.escapedText);
                                            propsFunctionList.push(escapedText);
                                            propsMorHandlers[this.getEventName(escapedText)] =
                                                this.filePathIdentifier;
                                        }
                                        else if (utils_1.typescript.isPropertyAssignment(item) &&
                                            utils_1.typescript.isIdentifier(item.name)) {
                                            const escapedText = String(item.name.escapedText);
                                            propsNormalList.push(escapedText);
                                            propsNormalHandlers[escapedText] = this.getPropsType(item.initializer);
                                        }
                                    }
                                    // 只有 props 传了函数的才执行以下流程
                                    if (propsFunctionList.length > 0) {
                                        // 修改 js 文件产物目录，其他三种基础文件在编译阶段再通过 setEntrySource 新增
                                        // 原因: setEntrySource 新增的 js 文件未经过 webpack 编译
                                        entryInfo.name = `${entryInfo.name}-${this.filePathIdentifier}`;
                                        this.needProcessList.push((_a = entryInfo === null || entryInfo === void 0 ? void 0 : entryInfo.entry) === null || _a === void 0 ? void 0 : _a.entryName);
                                        this.processPropsObj[(_b = entryInfo === null || entryInfo === void 0 ? void 0 : entryInfo.entry) === null || _b === void 0 ? void 0 : _b.entryName] = {
                                            normal: propsNormalList,
                                            function: propsFunctionList,
                                            normalHandlers: propsNormalHandlers,
                                            morHandlers: propsMorHandlers
                                        };
                                    }
                                }
                                utils_1.typescript.forEachChild(node, visitNode);
                            };
                            visitNode(sourceFile);
                        }
                    }
                    return entryInfo;
                });
                // 处理 xml、css 文件
                runner.hooks.postprocessorParser.tap(this.name, (fileContent, options) => {
                    var _a, _b, _c, _d, _e, _f, _g, _h, _j;
                    if (this.isNeedProcess(options)) {
                        if (((_a = options === null || options === void 0 ? void 0 : options.fileInfo) === null || _a === void 0 ? void 0 : _a.entryFileType) === utils_1.EntryFileType.template) {
                            this.addEntrySource(options);
                            const entryPathParse = path_1.default.parse(((_b = options === null || options === void 0 ? void 0 : options.fileInfo) === null || _b === void 0 ? void 0 : _b.entryName) || '');
                            const propsNormalList = ((_d = (_c = this.processPropsObj) === null || _c === void 0 ? void 0 : _c[`${entryPathParse.dir}/${entryPathParse.name}`]) === null || _d === void 0 ? void 0 : _d.normal) || [];
                            const propsFunctionList = ((_f = (_e = this.processPropsObj) === null || _e === void 0 ? void 0 : _e[`${entryPathParse.dir}/${entryPathParse.name}`]) === null || _f === void 0 ? void 0 : _f.function) || [];
                            const propsMorHandlers = ((_h = (_g = this.processPropsObj) === null || _g === void 0 ? void 0 : _g[`${entryPathParse.dir}/${entryPathParse.name}`]) === null || _h === void 0 ? void 0 : _h.morHandlers) || {};
                            // props 中的普通参数
                            const propsNormal = propsNormalList
                                .map((item) => {
                                return `${item}="{{${item}}}" `;
                            })
                                .join('');
                            // props 中的函数传参
                            const propsFunction = propsFunctionList
                                .map((item) => {
                                return `bind:${this.getEventName(item)}="$morEventHandlerProxy" `;
                            })
                                .join('');
                            const eventHandlerName = Buffer.from(JSON.stringify(propsMorHandlers)).toString('base64');
                            return `<mor-component ${propsNormal} ${propsFunction} data-mor-event-handlers="${eventHandlerName}"></mor-component>`;
                        }
                        else if (((_j = options === null || options === void 0 ? void 0 : options.fileInfo) === null || _j === void 0 ? void 0 : _j.entryFileType) === utils_1.EntryFileType.style) {
                            this.addEntrySource(options);
                            return ``;
                        }
                    }
                    return fileContent;
                });
                // 处理 js 文件
                runner.hooks.scriptParser.tap(this.name, (transformers, options) => {
                    var _a, _b, _c, _d, _e;
                    if (this.isNeedProcess(options)) {
                        const entryPathParse = path_1.default.parse(((_a = options === null || options === void 0 ? void 0 : options.fileInfo) === null || _a === void 0 ? void 0 : _a.entryName) || '');
                        const entryName = `${entryPathParse.dir}/${entryPathParse.name}${entryPathParse.ext}`;
                        const propsNormalList = ((_c = (_b = this.processPropsObj) === null || _b === void 0 ? void 0 : _b[`${entryPathParse.dir}/${entryPathParse.name}`]) === null || _c === void 0 ? void 0 : _c.normal) || [];
                        const propsNormalHandlers = ((_e = (_d = this.processPropsObj) === null || _d === void 0 ? void 0 : _d[`${entryPathParse.dir}/${entryPathParse.name}`]) === null || _e === void 0 ? void 0 : _e.normalHandlers) || {};
                        const properties = propsNormalList.map((item) => {
                            return `${item}: ${propsNormalHandlers[item]}`;
                        });
                        this.entryBuilder.setEntrySource(entryName, `Component({
  properties: {
    ${properties}
  },
  methods: {
    $morEventHandlerProxy(event) {
      const { detail } = event;
      if (detail.name) {
        this.triggerEvent(event.type, { ...detail.args[0] })
      }
    },
  },
})
`, 'additional');
                    }
                    return transformers;
                });
                // 处理 json 文件
                runner.hooks.configParser.tap(this.name, (config, options) => {
                    if (this.isNeedProcess(options)) {
                        this.addEntrySource(options);
                        return {
                            component: true,
                            usingComponents: {
                                'mor-component': `./index-${this.filePathIdentifier}`
                            }
                        };
                    }
                    return config;
                });
            }
        });
    }
    isNeedProcess(options) {
        var _a, _b, _c, _d, _e, _f, _g;
        const entryPathParse = path_1.default.parse(((_a = options === null || options === void 0 ? void 0 : options.fileInfo) === null || _a === void 0 ? void 0 : _a.entryName) || '');
        return (this.needProcessList.length > 0 &&
            ((_b = options === null || options === void 0 ? void 0 : options.fileInfo) === null || _b === void 0 ? void 0 : _b.entryName) &&
            ((_c = options === null || options === void 0 ? void 0 : options.fileInfo) === null || _c === void 0 ? void 0 : _c.entryType) === utils_1.EntryType.component &&
            (((_d = options === null || options === void 0 ? void 0 : options.fileInfo) === null || _d === void 0 ? void 0 : _d.entryFileType) === utils_1.EntryFileType.template ||
                ((_e = options === null || options === void 0 ? void 0 : options.fileInfo) === null || _e === void 0 ? void 0 : _e.entryFileType) === utils_1.EntryFileType.style ||
                ((_f = options === null || options === void 0 ? void 0 : options.fileInfo) === null || _f === void 0 ? void 0 : _f.entryFileType) === utils_1.EntryFileType.script ||
                ((_g = options === null || options === void 0 ? void 0 : options.fileInfo) === null || _g === void 0 ? void 0 : _g.entryFileType) === utils_1.EntryFileType.config) &&
            this.needProcessList.includes(`${entryPathParse.dir}/${entryPathParse.name}`));
    }
    addEntrySource(options) {
        var _a, _b;
        const entryPathParse = path_1.default.parse(((_a = options === null || options === void 0 ? void 0 : options.fileInfo) === null || _a === void 0 ? void 0 : _a.entryName) || '');
        const entryName = `${entryPathParse.dir}/${entryPathParse.name}-${this.filePathIdentifier}${entryPathParse.ext}`;
        this.entryBuilder.setEntrySource(entryName, (_b = options === null || options === void 0 ? void 0 : options.fileInfo) === null || _b === void 0 ? void 0 : _b.content, 'additional');
    }
    getEventName(attr) {
        // 判断小程序组件属性是否是事件（是否以on开头）
        const eventAttrReg = /^on([A-Za-z]+)/;
        const onMatch = attr.match(eventAttrReg);
        if (onMatch) {
            const eventName = onMatch[1];
            return utils_1.lodash.lowerFirst(eventName);
        }
        // 判断小程序组件属性是否是catch事件（是否以catch开头）
        const catchEventAttrReg = /^catch([A-Za-z]+)/;
        const catchMatch = attr.match(catchEventAttrReg);
        if (catchMatch) {
            const eventName = catchMatch[1];
            return utils_1.lodash.lowerFirst(eventName);
        }
        return '';
    }
    getPropsType(node) {
        if (utils_1.typescript.isStringLiteral(node))
            return `String`;
        if (utils_1.typescript.isNumericLiteral(node))
            return `Number`;
        if (utils_1.typescript.isObjectLiteralExpression(node))
            return `Object`;
        if (utils_1.typescript.isArrayLiteralExpression(node))
            return `Array`;
        if (utils_1.typescript.isArrowFunction(node) || utils_1.typescript.isFunctionExpression(node))
            return `Function`;
        if (!node.text && !node.properties && !node.elements && !node.parameters)
            return `Boolean`;
        return null;
    }
}
exports.ProcessComponentsPropsFunctionPlugin = ProcessComponentsPropsFunctionPlugin;
//# sourceMappingURL=processComponentsPropsFunctionPlugin.js.map